#!/usr/bin/env python
# coding: utf-8

'''
CLI script for opening the main location's door
'''
import argparse
import logging
import sys
from datetime import datetime
from pprint import pprint
from dateutil import tz
from dateutil.parser import parse as dateparse
from .nello import (LOGGER, Nello, NelloLoginException)

FIELD_LOCATION_ID = ['id', 'location_id']
FIELD_SHORT_ID = ['sid', 'short', 'short_loc_id', 'short_location_id']
FIELD_ADDRESS = ['addr', 'address', 'adress', 'addresse']
FIELDS = FIELD_LOCATION_ID + FIELD_SHORT_ID + FIELD_ADDRESS


def parse_args():
    '''
    Parse CLI args
    '''
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--username', required=True)
    parser.add_argument('-p', '--password', required=True)
    parser.add_argument(
        '-l', '--location', default=None,
        help='Target location ID (default: Main location)')
    parser.add_argument('-D', '--debug', action='store_true', default=False)
    subparsers = parser.add_subparsers(dest='action', help='Available actions')
    subparsers.required = True
    activity_parser = subparsers.add_parser('activity')
    activity_parser.add_argument(
        '-j', '--raw', action='store_true', default=False,
        help='Output RAW JSON')
    activity_parser.add_argument(
        '-r', '--reverse', action='store_true', default=False,
        help='Reverse output')
    subparsers.add_parser('open')
    list_parser = subparsers.add_parser('list')
    list_parser.add_argument(
        '-f', '--field', choices=FIELDS, help='Field filter')
    subparsers.add_parser('info')
    return parser.parse_args()


def get_target_location_id(nello, args):
    '''Get the target location ID.'''
    return args.location if args.location else \
        nello.main_location.location_id


def open_door(nello, target):
    '''
    Open the door.
    :param nello: Nello object
    :param target: Target Location ID
    '''
    if nello.open_door(target):
        print('Door opened successfully')
    else:
        print('Failed to open door')
        sys.exit(1)


def display_activity(nello, target, raw=False, reverse=False):
    '''
    Display the recent activity.
    :param nello: Nello object
    :param target: Target Location ID
    :param raw: Raw output mode
    :param reverse: Whether to reverse the activity
    '''
    activity = nello.get_activity(target)
    if reverse:
        activity = reversed(activity)
    if raw:
        pprint(list(activity))
    else:
        for act in activity:
            # Retrive raw date
            date = dateparse(act.get('date'))
            # Convert to local time
            utc_date = date.replace(tzinfo=tz.tzutc())
            localized_date = utc_date.astimezone(tz.tzlocal())
            date_str = datetime.strftime(
                localized_date, '%Y-%m-%d %H:%M:%S')
            print('[{}] {}'.format(
                date_str,
                act.get('description')))


def list_locations(nello, location=None, field=None):
    '''
    List the available locations
    :param nello: Nello object
    :param location: Only location to display
    :param field: Optional field filter
    '''
    all_locations = nello.locations
    locations = None
    if location:
        for loc in all_locations:
            if loc.location_id == location or loc.short_id == location:
                locations = [loc]
        if not locations:
            print('Could not find any location with ID: {}'.format(location),
                  file=sys.stderr)
            sys.exit(2)
    else:
        locations = all_locations

    for loc in locations:
        output = ''
        if field:
            req_filter = field.lower()
            if req_filter in FIELD_LOCATION_ID:
                output = loc.location_id
            elif req_filter in FIELD_SHORT_ID:
                output = loc.short_id
            elif req_filter in FIELD_ADDRESS:
                output = loc.address
        else:
            output = '# Location\n' \
                     'Location ID: {}\n' \
                     'Short Location ID: {}\n' \
                     'Address: {}'.format(
                         loc.location_id, loc.short_id, loc.address)
        print(output)


def display_info(nello):
    '''
    Display general info about the current account etc.
    '''
    print('User ID: {}\n'
          'Username: {}\n'
          'First name: {}\n'
          'Last name: {}\n'.format(
              nello.account.user_id,
              nello.account.username,
              nello.account.first_name,
              nello.account.last_name))
    print('Roles:')
    for role in nello.account.roles:
        is_admin = role.get('role') == 'unrestricted'
        print('- {}: {}{}'.format(
            role.get('location_id'),
            'Admin' if is_admin else 'User',
            ' (Inactive)' if not role.get('is_active', False) else ''))


def main():
    '''
    Main CLI function
    '''
    args = parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    LOGGER.debug('Args: %s', args)
    try:
        nello = Nello(username=args.username, password=args.password)
        LOGGER.debug('Nello: %s', vars(nello))
        if args.action == 'open':
            target = get_target_location_id(nello, args)
            open_door(nello, target)
        elif args.action == 'activity':
            target = get_target_location_id(nello, args)
            display_activity(nello, target, args.raw, args.reverse)
        elif args.action == 'list':
            list_locations(nello, location=args.location, field=args.field)
        elif args.action == 'info':
            display_info(nello)
    except NelloLoginException as exc:
        print(exc)
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding: utf-8

'''
Nello Intercom Library
Credit: https://forum.fhem.de/index.php/topic,75127.msg668871.html
'''

import logging
import requests
from .exceptions import (NelloLoginException, NelloTokenTimeoutException)
from .utils import (check_success, extract_error_message, extract_status_code)

LOGGER = logging.getLogger(__name__)


# pylint: disable=too-few-public-methods
class NelloObject(object):
    '''
    Base class for Nello Objects.
    '''

    def __init__(self, nello, json):
        self._nello = nello
        self._json = json


class NelloAccount(NelloObject):
    '''
    Nello Account information
    '''

    @property
    def user_id(self):
        '''
        User ID
        '''
        return self._json.get('user_id')

    @property
    def username(self):
        '''
        Username (email)
        '''
        return self._json.get('username')

    @property
    def first_name(self):
        '''
        First Name
        '''
        return self._json.get('first_name')

    @property
    def last_name(self):
        '''
        Last Name
        '''
        return self._json.get('last_name')

    @property
    def roles(self):
        '''
        Roles (permissions)
        Example:
        [{'home_ssid': None,
          'is_active': True,
          'location_id': 'XXX',
          'role': 'unrestricted',
          'stripe_id': 'XXX',
          'subscription_plan': ''}]
        '''
        return self._json.get('roles')


class NelloLocation(NelloObject):
    '''
    Class representation of a Nello Lock.
    '''

    @property
    def location_id(self):
        '''
        Location ID
        '''
        return self._json.get('location_id')

    @property
    def short_id(self):
        '''
        Location ID
        '''
        return self._json.get('short_loc_id')

    @property
    def address(self):
        '''
        Address of this location
        '''
        addr = self._json.get('address')
        if not addr:
            return
        country = addr.get('country')
        state = addr.get('state')
        city = addr.get('city')
        zip_code = addr.get('zip')
        street = addr.get('street')
        number = addr.get('number')
        german = country.lower() in ['deutschland', 'germany']
        return '{} {} {} {}{}, {}'.format(
            street if german else number,
            number if german else street,
            zip_code, city,
            '{}, '.format(state) if state != 'state' else '',
            country)

    @property
    def activity(self):
        '''
        Recent activity on this lock
        '''
        return self._nello.get_activity(self.location_id)

    def open_door(self):
        '''
        Open this lock
        '''
        return self._nello.open_door(self.location_id)

    def update(self):
        '''
        Update JSON data
        '''
        location_data = self._nello.get_locations()
        for loc in location_data.get('geofences'):
            if loc.get('location_id') == self.location_id:
                self._json = loc


class Nello(object):
    '''
    Nello Intercom API client
    '''
    def __init__(self, username, password):
        self.username = username
        self.password = password
        self._session = requests.Session()
        self._account = None

    @property
    def account(self):
        '''
        Account property. Will try to login if required.
        '''
        if not self._account:
            self.login()
        return self._account

    @property
    def locations(self):
        '''
        List of locations the current user has access to
        '''
        location_data = self.get_locations()
        locs = []
        for loc in location_data.get('geofences'):
            locs.append(NelloLocation(self, loc))
        return locs

    @property
    def main_location(self):
        '''
        Get the main location (lock)
        This equates to the first lock if there are multiple available
        '''
        all_locations = self.locations
        return self.locations[0] if all_locations else None

    def _request(self, method, path, json=None):
        '''
        Issue an API call.
        NOTE: This method does not try to log in again in case the
        authentication has expired. Use _retry_request() for this.
        :param method: HTTP method to use (GET or POST)
        :param path: URL path to the API object to call
        :param json: Optional JSON data
        '''
        LOGGER.debug('Method: %s, path: %s, json: %s', method, path, json)
        url = 'https://api.nello.io/{}'.format(path)
        LOGGER.debug('%s call to %s', method, url)
        LOGGER.debug('JSON Data: %s', json)
        response = self._session.request(method=method, url=url, json=json)
        response.raise_for_status()
        json_response = response.json()
        LOGGER.debug('JSON response: %s', json_response)
        if not check_success(json_response):
            status = extract_status_code(json_response)
            err_msg = extract_error_message(json_response)
            LOGGER.warning('JSON status: %s', status)
            LOGGER.warning('API Error message: %s', err_msg)
            if status == '400':
                raise NelloTokenTimeoutException(err_msg)
        return json_response

    def _retry_request(self, *args, **kwargs):
        '''
        Issue an API call that may required to log in again.
        :param method: HTTP method to use (GET or POST)
        :param path: URL path to the API object to call
        :param json: Optional JSON data
        '''
        # We never logged in. Let's do this now.
        if not self._account:
            self.login()
        try:
            json_response = self._request(*args, **kwargs)
        except NelloTokenTimeoutException:
            if self.login():
                json_response = self._request(*args, **kwargs)
        return json_response

    def login(self):
        '''
        Login to Nello server
        '''
        resp = self._request(
            method='POST',
            path='login',
            json={'username': self.username, 'password': self.password}
        )
        if not resp.get('authentication'):
            LOGGER.error('Authentication failed: %s', resp)
            err_msg = extract_error_message(resp)
            raise NelloLoginException('Login failed: {}'.format(err_msg))
        self._account = NelloAccount(self, resp.get('user'))
        LOGGER.info('Login successful. User ID: %s', self.account.user_id)
        return True

    def get_locations(self):
        '''
        Get all available locations
        '''
        return self._retry_request(method='GET', path='locations/')

    def get_activity(self, location_id=None):
        '''
        Get the activity log for a location
        '''
        if not location_id:
            location_id = self.main_location.location_id
        path = 'locations/{}/activity'.format(location_id)
        resp = self._retry_request(method='GET', path=path)
        return resp.get('activities')

    def open_door(self, location_id=None):
        '''
        Ring the buzzer AKA open the door
        :param location: Target location ID
        '''
        if not location_id:
            location_id = self.main_location.location_id
        path = 'locations/{}/users/{}/open'.format(
            location_id, self.account.user_id)
        resp = self._retry_request(
            method='POST',
            path=path,
            json={'type': 'swipe'}
        )
        return check_success(resp)



import os
from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='pynello',
    version='1.5.1',
    license='GPL3',
    description='Python library for nello.io intercoms',
    long_description=read('README.rst'),
    author='Philipp Schmitt',
    author_email='philipp@schmitt.co',
    url='https://github.com/pschmitt/pynello',
    packages=find_packages(),
    install_requires=['python-dateutil', 'requests'],
    entry_points={'console_scripts': ['nello=pynello.__main__:main']}
)



'''
Nello Exceptions
'''


class NelloException(Exception):
    '''
    Base Exception for Nello related errors
    '''
    pass


class NelloLoginException(NelloException):
    '''
    Exception to be raised when the login fails
    '''
    pass


class NelloTokenTimeoutException(NelloException):
    '''
    Exception to be raised when the auth ticket expires
    '''
    pass



'''
Misc utility functions
'''
import logging


LOGGER = logging.getLogger(__name__)


def extract_status_code(json_response):
    '''
    Extract the status code string from a JSON response
    '''
    return json_response.get('result', {}).get('status')


def extract_error_message(json_response):
    '''
    Extract the error message from an API call's JSON response
    '''
    return json_response.get('result', {}).get('message')


def check_success(json_response):
    '''
    Check whether an API call suceeded
    :param json_response: JSON response
    '''
    # The login call returns a status code of 'OK' when successful
    # The other methods should return '200'
    status_codes_ok = ['200', 'OK']
    status = extract_status_code(json_response)
    return status in status_codes_ok
